@@ -92,6 +92,8 @@ class JSONPath(Node):
 J = JSONPath()
 
 class JSONField(TextField):
+    field_type = 'JSON'
+
     def python_value(self, value):
         if value is not None:
             try:
@@ -211,6 +213,10 @@ class VirtualTableSchemaManager(SchemaManager):
         options = self.model.clean_options(
             merge_dict(self.model._meta.options, options))
 
+        # Structure:
+        # CREATE VIRTUAL TABLE <model>
+        # USING <extension_module>
+        # ([prefix_arguments, ...] fields, ... [arguments, ...], [options...])
         ctx = self._create_context()
         ctx.literal('CREATE VIRTUAL TABLE ')
         if safe:
@@ -924,7 +930,7 @@ def _sqlite_regexp(regex, value):
 class SqliteExtDatabase(SqliteDatabase):
     def __init__(self, database, c_extensions=None, rank_functions=True,
                  hash_functions=False, regexp_function=False,
-                 bloomfilter=False, *args, **kwargs):
+                 bloomfilter=False, json_contains=False, *args, **kwargs):
         super(SqliteExtDatabase, self).__init__(database, *args, **kwargs)
         self._row_factory = None
 
@@ -952,6 +958,8 @@ class SqliteExtDatabase(SqliteDatabase):
             if not prefer_c:
                 raise ValueError('C extension required to use bloomfilter.')
             register_bloomfilter(self)
+        if json_contains:
+            self.register_function(_json_contains, 'json_contains')
 
         self._c_extensions = prefer_c
 
@@ -1206,3 +1214,41 @@ def bm25(raw_match_info, *args):
             score += (idf * rhs) * weight
 
     return -score
+
+
+def _json_contains(src_json, obj_json):
+    stack = []
+    try:
+        stack.append((json.loads(obj_json), json.loads(src_json)))
+    except:
+        # Invalid JSON!
+        return False
+
+    while stack:
+        obj, src = stack.pop()
+        if isinstance(src, dict):
+            if isinstance(obj, dict):
+                for key in obj:
+                    if key not in src:
+                        return False
+                    stack.append((obj[key], src[key]))
+            elif isinstance(obj, list):
+                for item in obj:
+                    if item not in src:
+                        return False
+            elif obj not in src:
+                return False
+        elif isinstance(src, list):
+            if isinstance(obj, dict):
+                return False
+            elif isinstance(obj, list):
+                try:
+                    for i in range(len(obj)):
+                        stack.append((obj[i], src[i]))
+                except IndexError:
+                    return False
+            elif obj not in src:
+                return False
+        elif obj != src:
+            return False
+    return True
